@@ -6,7 +6,7 @@ import logging
 from datetime import datetime, timedelta
 import time
 
-__version_info__ = ('0', '0', '11')
+__version_info__ = ('0', '0', '12')
 __version__ = '.'.join(__version_info__)
 
 from src.VideoTrackingWindow import Ui_VideoTrackingWindow
@@ -28,6 +28,8 @@ import os
 import os.path as osp
 import numpy as np
 import cv2
+import skvideo
+import skvideo.io
 
 from PyQt5.QtGui import *
 from PyQt5.QtWidgets import *
@@ -36,39 +38,11 @@ from PyQt5.QtMultimedia import *
 from PyQt5.QtMultimediaWidgets import *
 
 
-class VideoSurfaceRects(QAbstractVideoSurface):
-    frameAvailable = pyqtSignal(QImage)
-
-    def __init__(self, widget: QWidget, parent: QObject):
-        super().__init__(parent)
-        self.widget = widget
-
-    def supportedPixelFormats(self, handleType):
-        return [QVideoFrame.Format_ARGB32, QVideoFrame.Format_ARGB32_Premultiplied,
-                QVideoFrame.Format_RGB32, QVideoFrame.Format_RGB24, QVideoFrame.Format_RGB565,
-                QVideoFrame.Format_RGB555, QVideoFrame.Format_ARGB8565_Premultiplied,
-                QVideoFrame.Format_BGRA32, QVideoFrame.Format_BGRA32_Premultiplied, QVideoFrame.Format_BGR32,
-                QVideoFrame.Format_BGR24, QVideoFrame.Format_BGR565, QVideoFrame.Format_BGR555,
-                QVideoFrame.Format_BGRA5658_Premultiplied, QVideoFrame.Format_AYUV444,
-                QVideoFrame.Format_AYUV444_Premultiplied, QVideoFrame.Format_YUV444,
-                QVideoFrame.Format_YUV420P, QVideoFrame.Format_YV12, QVideoFrame.Format_UYVY,
-                QVideoFrame.Format_YUYV, QVideoFrame.Format_NV12, QVideoFrame.Format_NV21,
-                QVideoFrame.Format_IMC1, QVideoFrame.Format_IMC2, QVideoFrame.Format_IMC3,
-                QVideoFrame.Format_IMC4, QVideoFrame.Format_Y8, QVideoFrame.Format_Y16,
-                QVideoFrame.Format_Jpeg, QVideoFrame.Format_CameraRaw, QVideoFrame.Format_AdobeDng]
-
-    def present(self, frame):
-        frame = frame.image()
-        self.widget.loadPixmap(QPixmap.fromImage(frame))
-        self.frameAvailable.emit(frame)
- 
-        return True
-
-
 
 class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
 
     rectAvailable = pyqtSignal()
+    changePixmapIndex = pyqtSignal(int)
 
     def __init__(self, parent=None):
         super(VideoTrackingWindow, self).__init__(parent)
@@ -77,6 +51,9 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         # uic.loadUi('VideoTrackingWindow.ui', self)
         self.filePath=None
 
+        self.labelCoordinates = QLabel('')
+        self.statusBar.addPermanentWidget(self.labelCoordinates)
+
         self.canvas = Canvas(parent=self)
         self.scrollArea.setWidget(self.canvas)        
         self.canvas.scrollRequest.connect(self.scrollRequest)
@@ -87,13 +64,15 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         # self.canvas.zoomRequest.connect(self.zoomNavig.zoomRequest)
         self.previous_shape = []
 
+
+        self.videolen = 0
         self.scrollBars = {
             Qt.Vertical: self.scrollArea.verticalScrollBar(),
             Qt.Horizontal: self.scrollArea.horizontalScrollBar()
         }
 
         # setup media player
-        self.mediaPlayer = QMediaPlayer(None, QMediaPlayer.VideoSurface)
+        # self.mediaPlayer = QMediaPlayer(None, QMediaPlayer.VideoSurface)
         self.videoProbe = QVideoProbe()
         # self.videoFrame = QVideoFrame()
 
@@ -111,26 +90,28 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
 
         # setup connection
         self.playButton.clicked.connect(self.playPause)
+        self.changePixmapIndex.connect(self.setPosition)
+
         self.nextButton.clicked.connect(self.nextFrame)
         self.prevButton.clicked.connect(self.prevFrame)
-        self.fastButton.clicked.connect(self.fastPlayback)
-        self.slowButton.clicked.connect(self.slowPlayback)
+        # self.fastButton.clicked.connect(self.fastPlayback)
+        # self.slowButton.clicked.connect(self.slowPlayback)
         self.positionSlider.sliderMoved.connect(self.setPosition)
         self.actionOpen_Video.triggered.connect(self.openVideo)
         self.actionSave_Annotation.triggered.connect(self.saveAnnotations)
         self.actionExit.triggered.connect(self.exitCall)
 
         # Setup the Media Player
-        self.videoSurface = VideoSurfaceRects(self.canvas, self)
-        self.mediaPlayer.setVideoOutput(self.videoSurface)
-        self.videoSurface.frameAvailable.connect(self.processFrame)
+        # self.videoSurface = VideoSurfaceRects(self.canvas, self)
+        # self.mediaPlayer.setVideoOutput(self.videoSurface)
+        # self.videoSurface.frameAvailable.connect(self.processFrame)
 
-        self.videoProbe.setSource(self.mediaPlayer)
+        # self.videoProbe.setSource(self.mediaPlayer)
         # self.videoProbe.videoFrameProbed.connect(self.processFrame)
-        self.mediaPlayer.stateChanged.connect(self.mediaStateChanged)
-        self.mediaPlayer.positionChanged.connect(self.positionChanged)
-        self.mediaPlayer.durationChanged.connect(self.durationChanged)
-        self.mediaPlayer.error.connect(self.handleError)
+        # self.mediaPlayer.stateChanged.connect(self.mediaStateChanged)
+        # self.mediaPlayer.positionChanged.connect(self.positionChanged)
+        # self.mediaPlayer.durationChanged.connect(self.durationChanged)
+        # self.mediaPlayer.error.connect(self.handleError)
 
         # Set Tracking
         # self.listTracker=[]
@@ -160,102 +141,194 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
 
         self.frame2pos = []
 
+        self.timer = QTimer(self)
+        self.timer.timeout.connect(self.nextFrame)
+
     @property
     def DeltaT(self):
         return 1000/self.FPS
 
+
+    @pyqtSlot()
     def playPause(self):
-        if self.isTracking():
-            self.playtracking = not self.playtracking
-            if self.playtracking:
-                self.playButton.setIcon(
-                    self.style().standardIcon(QStyle.SP_MediaPause))
-                self.nextFrame()
-
-                # self.mediaPlayer.setPosition(
-                #     int(self.mediaPlayer.position()+self.DeltaT))
-            else:
-                self.playButton.setIcon(
-                    self.style().standardIcon(QStyle.SP_MediaPlay))     
+        if self.timer.isActive():
+            self.pause()
         else:
-            if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
-                self.mediaPlayer.pause()
-            else:
-                self.mediaPlayer.play()
+            self.play()
 
-    def mediaStateChanged(self, state):
-        if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
-            self.playButton.setIcon(
-                self.style().standardIcon(QStyle.SP_MediaPause))
-        else:
-            self.playButton.setIcon(
-                self.style().standardIcon(QStyle.SP_MediaPlay))
+    def pause(self):
+        logging.info('Pause Streaming')
+        self.timer.stop()        
+        self.playButton.setIcon(
+            self.style().standardIcon(QStyle.SP_MediaPlay))
+        # self.pushButton_PlayPause.setText("Play")
 
-    def positionChanged(self, position):
-        if self.isTracking(self.prevPosition) != self.isTracking(position):
-            # print("hello!")
-            # if self.isTracking(position):
-            if self.playtracking:
-                self.playtracking = False
-                self.logging(
-                    f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [End of Tracking]")
-            # else:
-            if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
-                self.mediaPlayer.pause()
-                self.logging(
-                    f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [Start of Tracking]")
+    def play(self):
+        logging.info('Play Streaming')
+        self.timer.start(1000/self.FPS) #self.spinBox_FPS.value())        
+        self.playButton.setIcon(
+            self.style().standardIcon(QStyle.SP_MediaPause))
+        # self.pushButton_PlayPause.setText("Pause")
 
-            self.playButton.setIcon(
-                self.style().standardIcon(QStyle.SP_MediaPlay))
-            # return
-        self.positionSlider.setValue(position)
-        self.checkIfTracking.setChecked(self.isTracking())
-        # self.centralwidget.setEnabled(not self.isTracking())
-        # self.tabWidget.setCurrentIndex(int(self.isTracking()))
-        strPosition = str(timedelta(seconds=int(self.mediaPlayer.position()/1000)))
-        strDuration = str(timedelta(seconds=int(self.mediaPlayer.duration()/1000)))
-        self.timeLabel.setText(f"{strPosition}/{strDuration}")
-        self.prevPosition = position
+
+    # def playPause(self):
+    #     if self.isTracking() or True:
+    #         self.playtracking = not self.playtracking
+    #         if self.playtracking:
+    #             self.playButton.setIcon(
+    #                 self.style().standardIcon(QStyle.SP_MediaPause))
+    #             # self.nextFrame()
+
+    #             # self.mediaPlayer.setPosition(
+    #             #     int(self.mediaPlayer.position()+self.DeltaT))
+    #         else:
+    #             self.playButton.setIcon(
+    #                 self.style().standardIcon(QStyle.SP_MediaPlay))     
+        # else:
+        #     if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
+        #         self.mediaPlayer.pause()
+        #     else:
+        #         self.mediaPlayer.play()
+
+    # def mediaStateChanged(self, state):
+    #     if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
+    #         self.playButton.setIcon(
+    #             self.style().standardIcon(QStyle.SP_MediaPause))
+    #     else:
+    #         self.playButton.setIcon(
+    #             self.style().standardIcon(QStyle.SP_MediaPlay))
+
+    # def positionChanged(self, position):
+    #     if self.isTracking(self.prevPosition) != self.isTracking(position):
+    #         # print("hello!")
+    #         # if self.isTracking(position):
+    #         if self.playtracking:
+    #             self.playtracking = False
+    #             self.logging(
+    #                 f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [End of Tracking]")
+    #         # else:
+    #         if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
+    #             self.mediaPlayer.pause()
+    #             self.logging(
+    #                 f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [Start of Tracking]")
+
+    #         self.playButton.setIcon(
+    #             self.style().standardIcon(QStyle.SP_MediaPlay))
+    #         # return
+    #     self.positionSlider.setValue(position)
+    #     self.checkIfTracking.setChecked(self.isTracking())
+    #     # self.centralwidget.setEnabled(not self.isTracking())
+    #     # self.tabWidget.setCurrentIndex(int(self.isTracking()))
+    #     strPosition = str(timedelta(seconds=int(self.mediaPlayer.position()/1000)))
+    #     strDuration = str(timedelta(seconds=int(self.mediaPlayer.duration()/1000)))
+    #     self.timeLabel.setText(f"{strPosition}/{strDuration}")
+    #     self.prevPosition = position
         
 
-    def durationChanged(self, duration):
-        self.positionSlider.setRange(0, duration)
-        # self.positionSlider.setTickInterval(1000)
+    # def durationChanged(self, duration):
+    #     self.positionSlider.setRange(0, duration)
+    #     # self.positionSlider.setTickInterval(1000)
+
+    #     self.frame2pos = [0]
+    #     t = 0
+    #     while t < duration:
+    #         t += self.DeltaT
+    #         self.frame2pos.append(int(t))
+
+    def setPosition(self, index):
+        if index < 0:
+             index = 0
+
+        if index >= self.videolen:
+            self.pause()
+            self.tracking = False
+            index = self.videolen-1
 
-        self.frame2pos = [0]
-        t = 0
-        while t < duration:
-            t += self.DeltaT
-            self.frame2pos.append(int(t))
 
-    def setPosition(self, position):
         self.previous_shape = self.canvas.shapes
 
-        if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
-            self.mediaPlayer.pause()
+        self.frame_index = index
 
-        self.mediaPlayer.setPosition(int(position/self.DeltaT)*self.DeltaT)
-        self.logging(f"Position set at time: {self.mediaPlayer.position()/1000:.3f} s")
+        self.positionSlider.setValue(index)
+        self.positionSlider.setToolTip(f"Frame {index}/{self.videolen}")
+        self.timeLabel.setText(f"Frame {index}/{self.videolen}")
+
+        # self.cap.set(2,index);
+        # self.rgbImage_vd = self.videoFrames[self.frame_index,:,:,:] # = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
+        self.rgbImage = self.videoFrames[index]
+        # self.shapes = 
+        # self.prev_shapes = self.shapes
+        self.shapes = self.videoLabels[index]
+
+        h, w, ch = self.rgbImage.shape
+        bytesPerLine = ch * w
+        convertToQtFormat = QImage(self.rgbImage.data, w, h, bytesPerLine, QImage.Format_RGB888)
+        self.QImage = convertToQtFormat #.scaled(640, 480, Qt.KeepAspectRatio)
+        # self.label_videoframe.setPixmap(QPixmap.fromImage(self.QImage))
+        
+        self.canvas.loadPixmap(QPixmap.fromImage(self.QImage))
+        # if self.labelList
+        # print(self.shapes)
+        self.canvas.loadShapes(self.shapes)
+
+        if self.isTracking():
+            self.processFrame(self.rgbImage)
+
+        # if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
+        #     self.mediaPlayer.pause()
+
+        # self.mediaPlayer.setPosition(int(position/self.DeltaT)*self.DeltaT)
+
+    #     if self.isTracking(self.prevPosition) != self.isTracking(position):
+    #         # print("hello!")
+    #         # if self.isTracking(position):
+    #         if self.playtracking:
+    #             self.playtracking = False
+    #             self.logging(
+    #                 f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [End of Tracking]")
+    #         # else:
+    #         if self.mediaPlayer.state() == QMediaPlayer.PlayingState:
+    #             self.mediaPlayer.pause()
+    #             self.logging(
+    #                 f"Paused at time: {self.mediaPlayer.position()/1000:.3f} s [Start of Tracking]")
+
+    #         self.playButton.setIcon(
+    #             self.style().standardIcon(QStyle.SP_MediaPlay))
+    #         # return
+    #     self.positionSlider.setValue(position)
+    #     self.checkIfTracking.setChecked(self.isTracking())
+    #     # self.centralwidget.setEnabled(not self.isTracking())
+    #     # self.tabWidget.setCurrentIndex(int(self.isTracking()))
+    #     strPosition = str(timedelta(seconds=int(self.mediaPlayer.position()/1000)))
+    #     strDuration = str(timedelta(seconds=int(self.mediaPlayer.duration()/1000)))
+    #     self.timeLabel.setText(f"{strPosition}/{strDuration}")
+    #     self.prevPosition = position
+    
+        self.logging(f"Position set at time: {index*self.DeltaT/1000:.3f} s")
         # print("pos in frame?", self.mediaPlayer.position() in self.frame2pos)
 
     def nextFrame(self):
-        self.setPosition(
-            int(self.mediaPlayer.position()+self.DeltaT*1.5))
+        self.changePixmapIndex.emit(self.frame_index+1)
+
+        # self.setPosition(
+        #     int(self.mediaPlayer.position()+self.DeltaT*1.5))
 
     def prevFrame(self):
+        self.changePixmapIndex.emit(self.frame_index-1)
 
-        self.setPosition(
-            int(self.mediaPlayer.position()-self.DeltaT))
 
-    def fastPlayback(self):
-        self.mediaPlayer.setPlaybackRate(self.mediaPlayer.playbackRate()*2.0)
-        self.logging(
-            f"Playback speed set to {self.mediaPlayer.playbackRate()}x")
+        # self.setPosition(
+        #     int(self.mediaPlayer.position()-self.DeltaT))
 
-    def slowPlayback(self):
-        self.mediaPlayer.setPlaybackRate(self.mediaPlayer.playbackRate()/2.0)
-        self.logging(
-            f"Playback speed set to {self.mediaPlayer.playbackRate()}x")
+    # def fastPlayback(self):
+    #     self.mediaPlayer.setPlaybackRate(self.mediaPlayer.playbackRate()*2.0)
+    #     self.logging(
+    #         f"Playback speed set to {self.mediaPlayer.playbackRate()}x")
+
+    # def slowPlayback(self):
+    #     self.mediaPlayer.setPlaybackRate(self.mediaPlayer.playbackRate()/2.0)
+    #     self.logging(
+    #         f"Playback speed set to {self.mediaPlayer.playbackRate()}x")
 
     def handleError(self):
         self.centralwidget.setEnabled(False)
@@ -277,13 +350,47 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
             self.FPS = cap.get(cv2.CAP_PROP_FPS)
             self.logging(f"FPS read: {self.FPS} fps")
             # read Media
-            self.mediaPlayer.setMedia(
-                QMediaContent(QUrl.fromLocalFile(filename)))
+            # self.mediaPlayer.setMedia(
+            #     QMediaContent(QUrl.fromLocalFile(filename)))
             self.centralwidget.setEnabled(True)
             self.logging(f"Video {filename} opened")
 
             # self.mediaPlayer.play()
-            self.mediaPlayer.pause()
+            # self.mediaPlayer.pause()
+            self.readVideo(filename)
+    
+    def readVideo(self, path):
+        self.filePath = path
+        import time
+        start_time = time.time()
+        videogen = skvideo.io.FFmpegReader(path)
+        (numFiles, _, _, _) = videogen.getShape()
+        # QProgressDialog progress("Copying files...", "Abort Copy", 0, numFiles, this);
+        progress = QProgressDialog("Reading frames...", "Abort Reading", 0, numFiles, self)
+        progress.setWindowModality(Qt.WindowModal)
+
+        self.videoFrames = []
+        self.videoLabels = []
+        for i, frame in enumerate(videogen):
+
+            progress.setValue(i)
+
+            if progress.wasCanceled() or i > 200:
+                break
+
+            self.videoFrames.append(frame)
+            self.videoLabels.append([])
+        
+        progress.setValue(numFiles)
+
+        self.videolen = len(self.videoFrames)
+
+        elapsed_time = time.time() - start_time
+        logging.info(f'Opening Video: {self.videolen} frames read in {elapsed_time:.3f}s')
+        
+        self.positionSlider.setMaximum(self.videolen-1)
+        self.positionSlider.setMinimum(0)
+        self.changePixmapIndex.emit(0)
 
 
     def saveAnnotations(self, filename=False):
@@ -324,7 +431,8 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
     def isTracking(self, pos = None):
         # print("isTracking?")
         if pos is None:
-            pos = self.mediaPlayer.position()
+            pos = self.frame_index
+            # pos = self. self.mediaPlayer.position()
 
         for i in range(self.listTracker.count()):
             this_item = self.listTracker.item(i)
@@ -334,14 +442,14 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         return False
 
     def processFrame(self, videoFrame):
-        currentPos = self.mediaPlayer.position()
+        currentPos = self.frame_index
 
 
         self.canvas.shapes
 
-        self.qimage = videoFrame.convertToFormat(4)
-        if self.qimage.bits() is None:
-            return
+        # self.qimage = videoFrame.convertToFormat(4)
+        # if self.qimage.bits() is None:
+        #     return
             
         new_shapes = []
 
@@ -371,15 +479,15 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
                     print("already tracked!")
                     continue                    
 
-                new_box = this_tracker.run(self.qimage, currentPos)
+                new_box = this_tracker.run(self.rgbImage, currentPos)
 
                 if new_box.topLeft == new_box.bottomRight:
                     # self.listTracker.repaint()
                     self.playtracking = False
                     # self.SP_MediaPlay.
-
-                    self.playButton.setIcon(
-                        self.style().standardIcon(QStyle.SP_MediaPlay))
+                    self.pause()
+                    # self.playButton.setIcon(
+                    #     self.style().standardIcon(QStyle.SP_MediaPlay))
                     continue
 
                 print(new_box)
@@ -414,12 +522,12 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         
         current_item = self.listTracker.currentItem()
         current_tracker = self.listTracker.itemWidget(current_item)
-        if current_tracker.isTracking(self.mediaPlayer.position()):
-            current_tracker.time_stop = self.mediaPlayer.position()
+        if current_tracker.isTracking(self.frame_index):
+            current_tracker.time_stop = self.frame_index
             current_tracker.updateItem()
             # self.centralwidget.setEnabled(not self.isTracking())
             # self.tabWidget.setCurrentIndex(int(self.isTracking()))
-            self.positionChanged(self.mediaPlayer.position())
+            # self.positionChanged(self.mediaPlayer.position())
         else:
             self.logging("Tracker already done")
 
@@ -439,7 +547,7 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         # current_tracker = self.listTracker.itemWidget(current_item)
     
     def addTrackerClicked(self):
-        self.mediaPlayer.pause()
+        # self.mediaPlayer.pause()
         self.canvas.setEditing(False)
 
 
@@ -478,10 +586,10 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
         # self.shapesToItems[shape] = item
         
         # qimage = self.videoSurface.videoFrame_QImage.convertToFormat(4)
-        ptr = self.qimage.bits()
-        ptr.setsize(self.qimage.byteCount())
-        img = np.array(ptr).reshape(
-            self.qimage.height(), self.qimage.width(), 4)  # Copies the data
+        # ptr = self.qimage.bits()
+        # ptr.setsize(self.qimage.byteCount())
+        # img = np.array(ptr).reshape(
+        #     self.qimage.height(), self.qimage.width(), 4)  # Copies the data
 
         # display_name = f'Display: Draw Initial Bounding Box'
         # cv2.imshow(display_name, img)
@@ -506,9 +614,9 @@ class VideoTrackingWindow(QMainWindow, Ui_VideoTrackingWindow):
 
         # # print(self.grabber.rects)
         init_state = shape2box(shape)
-        new_tracker = Tracker(time_start=self.mediaPlayer.position(),
-                              time_stop=self.mediaPlayer.duration(),
-                              init_QImage=self.qimage,
+        new_tracker = Tracker(time_start=self.frame_index, #self.mediaPlayer.position(),
+                              time_stop=self.videolen,  # self.mediaPlayer.duration(),
+                              init_CVImage=self.rgbImage,
                               init_box=init_state,
                               FPS=self.FPS)
 
@@ -569,7 +677,7 @@ OPENCV_OBJECT_TRACKERS = {
 
 
 class Tracker(QWidget):
-    def __init__(self, time_start, time_stop, init_box, init_QImage, FPS, parent=None):
+    def __init__(self, time_start, time_stop, init_box, init_CVImage, FPS, parent=None):
         super(Tracker, self).__init__(parent)
         self.time_start = time_start
         # print(time_start)
@@ -578,13 +686,18 @@ class Tracker(QWidget):
         self.FPS = FPS
         self.trackingAlgorithm = None
         # self.boxes = {f"{self.time_start}": self.box2rect(init_box)}
-        # self.frames = {f"{self.time_start}": init_QImage}
+        # self.frames = {f"{self.time_start}": init_CVImage}
 
         self.boxes = [self.box2rect(init_box)]
-        self.frames = [self.QImage2CVImage(init_QImage)]
+        # self.frames = [self.QImage2CVImage(init_CVImage)]
+        self.frames = [init_CVImage]
         self.times = [self.time_start]
 
-        pixmap = QPixmap.fromImage(init_QImage)
+        h, w, ch = self.frames[-1].shape
+        bytesPerLine = ch * w
+        convertToQtFormat = QImage(self.frames[-1].data, w, h, bytesPerLine, QImage.Format_RGB888)
+        # self.QImage = convertToQtFormat
+        pixmap = QPixmap.fromImage(convertToQtFormat)
         pixmap = pixmap.copy(self.boxes[-1])
         pixmap = pixmap.scaled(20, 20, QtCore.Qt.KeepAspectRatio)        
         self.iconQLabel = QLabel()
@@ -623,7 +736,7 @@ class Tracker(QWidget):
         # if qimage is None:
         #     print("self.qimage is none", qimage)
         #     return self.box2rect((0.0,0.0,0.0,0.0))
-        img = self.QImage2CVImage(qimage)[:, :, :3]
+        img = qimage #self.QImage2CVImage(qimage)[:, :, :3]
         # ptr = qimage.bits()
         # ptr.setsize(qimage.byteCount())
         # img = np.array(ptr).reshape(
@@ -705,7 +818,7 @@ class Tracker(QWidget):
 
 
     def getName(self):
-        return f"From {self.time_start/1000}s to {self.time_stop/1000}s"
+        return f"From {self.time_start/1000}s to {self.time_stop} frames"
 
     def addBox(self, box):
         self.boxes.append(box)
